"""
addons/eventscripts/mysqlads/mysqlads.py
Version 1.00.000
(C) [#OMEGA] - K2

IMPORTANT

This plugin is designed to be used with
MySQLdb

Agreement:
 
By using this file you agree to the following terms:

This file is delivered 'as-is'. 
I will not take resposibilty for any damage resulting of the use or not-use of this file. 

As for sharing permissions:
http://creativecommons.org/licenses/by-nc-nd/3.0/

If wish to alter the file/contents or reuse some parts please contact me.
"""

# ######## #
# Imports  #
# ######## #

# ES Libs
import es
import gamethread
import playerlib
import langlib
import usermsg

# Custom ES Libs
import pubcvarlib

# Python Libs
from configobj import ConfigObj
import re
import urllib2
import time
import random

# Custom Python Libs
import MySQLdb

# ########## #
# Addon Info #
# ########## #

info = es.AddonInfo()
info.name        = "MySQL Advertisements"
info.version     = "1.00.000"
info.author      = "[#OMEGA] - K2"
info.url         = "http://www.german-slaughterhouse.de"
info.description = "MySQL Advertisements"
info.tags        = "mysqlads mysql advertisements serverads"

# ########### #
# Global Data #
# ########### #

# Load the Config
config = ConfigObj(es.getAddonPath("mysqlads") + '/config.ini')

# Load the Translation/String file
lang = langlib.Strings(es.getAddonPath("mysqlads") + '/mysqlads-strings.ini')	

# ############# #
# Load & Unload #
# ############# #

def load():
	# Set Pub vars
	pubcvarlib.Register('mysqlads_ver', info.version)
	
	if config['MySQL']['sid'] == '-1':
		# Get IP and Port
		global ip,port
		
		port = es.getString('hostport')
		
		ip = re.search("[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}",urllib2.urlopen('http://www.whatismyip.de').read()).group(0)
	
	# Init DB
	global db
	db = Database()
	
	if db.data['servers']:
		gamethread.delayedname(db.data['servers']['interval'], 'mysqlads_adloop', AdLoop) 
	
def unload():
	# Cleanup pub vars
	pubcvarlib.Unregister('mysqlads_ver')
	
	# Disconnect from DB
	db.Disconnect()
	
	#
	gamethread.cancelDelayed('mysqlads_adloop') 

#
# Events
#

def es_map_start(ev):
	db.SyncDB()
	# Restart the Loop
	gamethread.cancelDelayed('mysqlads_adloop') 
	gamethread.delayedname(db.data['servers']['interval'], 'mysqlads_adloop', AdLoop) 

#
# Classes
#	
	
class Database(object):
	def __init__(self):
		self.con = MySQLdb.connect(config['MySQL']['server'], config['MySQL']['user'], config['MySQL']['password'], config['MySQL']['database'])
		self.cur = self.con.cursor()
		
		# Create Tables
		
		# Value Sizes
		# TINYINT 255
		# SMALLINT 65535
		# MEDIUMINT 16777215
		# INT 4294967295
		self.cur.execute("""
			CREATE TABLE IF NOT EXISTS %sadvertisements (
				`aid` MEDIUMINT UNSIGNED PRIMARY KEY AUTO_INCREMENT NOT NULL,
				`display` TINYINT UNSIGNED NOT NULL DEFAULT 1,
				`servers` TEXT NOT NULL,
				`order` TINYINT UNSIGNED NOT NULL DEFAULT 128
			);
		""" % (config['MySQL']['prefix']))
		
		self.cur.execute("""
			CREATE TABLE IF NOT EXISTS %stranslations (
			`aid` MEDIUMINT UNSIGNED NOT NULL,
			`lang` CHAR(2) NOT NULL DEFAULT 'en',
			`text` TEXT NOT NULL
			);
		""" % (config['MySQL']['prefix']))
		
		self.cur.execute("""
			CREATE TABLE IF NOT EXISTS %sservers (
				`sid` TINYINT UNSIGNED PRIMARY KEY AUTO_INCREMENT NOT NULL,
				`ip` CHAR(15) NOT NULL,
				`port` SMALLINT UNSIGNED NOT NULL,
				`interval` SMALLINT UNSIGNED NOT NULL DEFAULT 30,
				`dateformat` CHAR(10) NOT NULL DEFAULT '%%d.%%m.%%Y',
				`timeformat` CHAR(10) NOT NULL DEFAULT '%%H:%%M:%%S',
				`random` BOOLEAN NOT NULL DEFAULT FALSE
			);
		""" % (config['MySQL']['prefix']))
		
		# Syncronize DB
		self.SyncDB()
	def SyncDB(self):
		Debug('Starting database syncronisation!')
		self.data = {}
		self.data['servers'] = {}
		self.data['advertisements'] = {}
		self.data['advertisements']['list'] = []
		
		# Get ServerID and configuration vars
		if config['MySQL']['sid'] == '-1':
			global ip, port
			self.cur.execute('SELECT `sid`, `interval`, `dateformat`, `timeformat`, `random` FROM %sservers WHERE `ip` = "%s" AND `port` = "%s" LIMIT 1' % (config['MySQL']['prefix'], ip, port))
		else:
			self.cur.execute('SELECT `sid`, `interval`, `dateformat`, `timeformat`, `random` FROM %sservers WHERE `sid` = "%s" LIMIT 1' % (config['MySQL']['prefix'], config['MySQL']['sid']))
		data = self.cur.fetchone()
		if data:
			self.data['servers']['sid'] = data[0]
			self.data['servers']['interval'] = data[1]
			self.data['servers']['dateformat'] = data[2]
			self.data['servers']['timeformat'] = data[3]
			self.data['servers']['random'] = data[4]
			# Get all valid advertisements
			self.cur.execute("""
				SELECT `aid`, `display`
				FROM %sadvertisements
				WHERE LOCATE('-1,', servers) != 0 OR LOCATE('%s,',servers) != 0
				ORDER BY `order`
				""" % (config['MySQL']['prefix'],self.data['servers']['sid']))
				
			data = self.cur.fetchall()
			
			for item in data:
				self.data['advertisements']['list'].append(item[0])
				self.data['advertisements'][item[0]] = {}
				self.data['advertisements'][item[0]]['display'] = item[1]
				self.data['advertisements'][item[0]]['translations'] = {}
				# Get all corresponding translations
				self.cur.execute("SELECT lang, text FROM %stranslations WHERE aid = '%s'" % (config['MySQL']['prefix'], item[0]))
				tdata = self.cur.fetchall()
				for translation in tdata:
					self.data['advertisements'][item[0]]['translations'][translation[0]] = translation[1]
			Debug('Database syncronisation completed!')
		else:
			Debug('Server not found in database. Unloading.')
			es.unload('mysqlads')
	def Disconnect(self):
		self.cur.close()
		self.con.close()
		
#
# Functions
#

def AdLoop(aid=-1):
	if db.data['servers']['random']:
		aid = db.data['advertisements']['list'][random.randint(0,len(db.data['advertisements']['list'])) - 1]
	else:
		if aid == -1:
			index = 0
		else:
			index = db.data['advertisements']['list'].index(aid) + 1
			if index >= len(db.data['advertisements']['list']):
				index = 0
		aid = db.data['advertisements']['list'][index]
	DisplayAd(aid)
	gamethread.delayedname(db.data['servers']['interval'], 'mysqlads_adloop', AdLoop, aid) 
	
def DisplayAd(id):
	"""
	Modes
	1 = Chat 
	2 = Menu
	4 = Hint
	8 = Center
	"""
	mode = db.data['advertisements'][id]['display']
	if mode:
		for userid in es.getUseridList():
			lng = playerlib.getPlayer(userid).get('lang')
			
			# Any valid message for this achievement?
			if db.data['advertisements'][id]['translations'].has_key(lng):
				text = db.data['advertisements'][id]['translations'][lng]
			elif db.data['advertisements'][id]['translations'].has_key('en'):
				text = db.data['advertisements'][id]['translations']['en']
			else:
				text = lang('missing translation',{'aid': id},lng)
			
			# Parse variables
			if '{CURRENTMAP}' in text:
				text = text.replace('{CURRENTMAP}',es.getString('eventscripts_currentmap'))
			if '{DATE}' in text:
				text = text.replace('{DATE}',time.strftime(db.data['servers']['dateformat']))
			if '{TIME}' in text:
				text = text.replace('{TIME}',time.strftime(db.data['servers']['timeformat']))
			cvars = re.findall(r'\{CVAR:[^\}]*\}',text)
			if cvars:
				for cvar in cvars:
					text = text.replace(cvar,es.getString(cvars[cvars.index(cvar)][6:-1]))
			
			if mode & 1:
				es.tell(userid,'#multi',text)
			# Following modes don't use the chat colours
			for color in ['#green','#lightgreen','#default']:
				matches = re.findall(color, text, re.IGNORECASE)
				for match in matches:
					text = text.replace(match,'')
			if mode & 2:
				es.menu(10,userid,text)
			if mode & 4:
				usermsg.hudhint(userid,text)
			if mode & 8:
				usermsg.centermsg(userid,text)
			
def Debug(msg):
	es.dbgmsg(0,'mysqlads.py: ' + msg)